@@ -9,6 +9,7 @@ import inspect
 import json
 import os
 import re
+import codecs
 from functools import wraps
 
 try:
@@ -18,7 +19,7 @@ except ImportError:  # pragma: no cover
 else:
     _have_yaml = True
 
-__version__ = '1.1.0'
+__version__ = '1.2.1'
 
 # These attributes will not conflict with any real python attribute
 # They are added to the decorated test method and processed later
@@ -27,6 +28,7 @@ __version__ = '1.1.0'
 DATA_ATTR = '%values'      # store the data the test must run with
 FILE_ATTR = '%file_path'   # store the path to JSON file
 UNPACK_ATTR = '%unpack'    # remember that we have to unpack values
+index_len = 5              # default max length of case index
 
 
 try:
@@ -58,9 +60,21 @@ def data(*values):
 
     Should be added to methods of instances of ``unittest.TestCase``.
 
+    """
+    global index_len
+    index_len = len(str(len(values)))
+    return idata(values)
+
+
+def idata(iterable):
+    """
+    Method decorator to add to your test methods.
+
+    Should be added to methods of instances of ``unittest.TestCase``.
+
     """
     def wrapper(func):
-        setattr(func, DATA_ATTR, values)
+        setattr(func, DATA_ATTR, iterable)
         return func
     return wrapper
 
@@ -108,18 +122,20 @@ def mk_test_name(name, value, index=0):
     only of trivial values.
     """
 
+    # Add zeros before index to keep order
+    index = "{0:0{1}}".format(index + 1, index_len)
     if not is_trivial(value):
-        return "{0}_{1}".format(name, index + 1)
+        return "{0}_{1}".format(name, index)
     try:
         value = str(value)
     except UnicodeEncodeError:
         # fallback for python2
         value = value.encode('ascii', 'backslashreplace')
-    test_name = "{0}_{1}_{2}".format(name, index + 1, value)
-    return re.sub('\W|^(?=\d)', '_', test_name)
+    test_name = "{0}_{1}_{2}".format(name, index, value)
+    return re.sub(r'\W|^(?=\d)', '_', test_name)
 
 
-def feed_data(func, new_name, *args, **kwargs):
+def feed_data(func, new_name, test_data_docstring, *args, **kwargs):
     """
     This internal method decorator feeds the test data item to the test.
 
@@ -128,19 +144,25 @@ def feed_data(func, new_name, *args, **kwargs):
     def wrapper(self):
         return func(self, *args, **kwargs)
     wrapper.__name__ = new_name
-    # Try to call format on the docstring
-    if func.__doc__:
-        try:
-            wrapper.__doc__ = func.__doc__.format(*args, **kwargs)
-        except (IndexError, KeyError):
-            # Maybe the user has added some of the formating strings
-            # unintentionally in the docstring. Do not raise an exception as it
-            # could be that he is not aware of the formating feature.
-            pass
+    wrapper.__wrapped__ = func
+    # set docstring if exists
+    if test_data_docstring is not None:
+        wrapper.__doc__ = test_data_docstring
+    else:
+        # Try to call format on the docstring
+        if func.__doc__:
+            try:
+                wrapper.__doc__ = func.__doc__.format(*args, **kwargs)
+            except (IndexError, KeyError):
+                # Maybe the user has added some of the formating strings
+                # unintentionally in the docstring. Do not raise an exception
+                # as it could be that user is not aware of the
+                # formating feature.
+                pass
     return wrapper
 
 
-def add_test(cls, test_name, func, *args, **kwargs):
+def add_test(cls, test_name, test_docstring, func, *args, **kwargs):
     """
     Add a test case to this class.
 
@@ -148,13 +170,13 @@ def add_test(cls, test_name, func, *args, **kwargs):
     name.
 
     """
-    setattr(cls, test_name, feed_data(func, test_name, *args, **kwargs))
+    setattr(cls, test_name, feed_data(func, test_name, test_docstring,
+            *args, **kwargs))
 
 
 def process_file_data(cls, name, func, file_attr):
     """
     Process the parameter in the `file_data` decorator.
-
     """
     cls_path = os.path.abspath(inspect.getsourcefile(cls))
     data_file_path = os.path.join(os.path.dirname(cls_path), file_attr)
@@ -167,7 +189,9 @@ def process_file_data(cls, name, func, file_attr):
     # If file does not exist, provide an error function instead
     if not os.path.exists(data_file_path):
         test_name = mk_test_name(name, "error")
-        add_test(cls, test_name, create_error_func("%s does not exist"), None)
+        test_docstring = """Error!"""
+        add_test(cls, test_name, test_docstring,
+                 create_error_func("%s does not exist"), None)
         return
 
     _is_yaml_file = data_file_path.endswith((".yml", ".yaml"))
@@ -175,15 +199,17 @@ def process_file_data(cls, name, func, file_attr):
     # Don't have YAML but want to use YAML file.
     if _is_yaml_file and not _have_yaml:
         test_name = mk_test_name(name, "error")
+        test_docstring = """Error!"""
         add_test(
             cls,
             test_name,
+            test_docstring,
             create_error_func("%s is a YAML file, please install PyYAML"),
             None
         )
         return
 
-    with open(data_file_path) as f:
+    with codecs.open(data_file_path, 'r', 'utf-8') as f:
         # Load the data from YAML or JSON
         if _is_yaml_file:
             data = yaml.safe_load(f)
@@ -196,7 +222,6 @@ def process_file_data(cls, name, func, file_attr):
 def _add_tests_from_data(cls, name, func, data):
     """
     Add tests from data loaded from the data file into the class
-
     """
     for i, elem in enumerate(data):
         if isinstance(data, dict):
@@ -206,9 +231,26 @@ def _add_tests_from_data(cls, name, func, data):
             value = elem
             test_name = mk_test_name(name, value, i)
         if isinstance(value, dict):
-            add_test(cls, test_name, func, **value)
+            add_test(cls, test_name, test_name, func, **value)
         else:
-            add_test(cls, test_name, func, value)
+            add_test(cls, test_name, test_name, func, value)
+
+
+def _is_primitive(obj):
+    """Finds out if the obj is a "primitive". It is somewhat hacky but it works.
+    """
+    return not hasattr(obj, '__dict__')
+
+
+def _get_test_data_docstring(func, value):
+    """Returns a docstring based on the following resolution strategy:
+    1. Passed value is not a "primitive" and has a docstring, then use it.
+    2. In all other cases return None, i.e the test name is used.
+    """
+    if not _is_primitive(value) and value.__doc__:
+        return value.__doc__
+    else:
+        return None
 
 
 def ddt(cls):
@@ -239,14 +281,27 @@ def ddt(cls):
         if hasattr(func, DATA_ATTR):
             for i, v in enumerate(getattr(func, DATA_ATTR)):
                 test_name = mk_test_name(name, getattr(v, "__name__", v), i)
+                test_data_docstring = _get_test_data_docstring(func, v)
                 if hasattr(func, UNPACK_ATTR):
                     if isinstance(v, tuple) or isinstance(v, list):
-                        add_test(cls, test_name, func, *v)
+                        add_test(
+                            cls,
+                            test_name,
+                            test_data_docstring,
+                            func,
+                            *v
+                        )
                     else:
                         # unpack dictionary
-                        add_test(cls, test_name, func, **v)
+                        add_test(
+                            cls,
+                            test_name,
+                            test_data_docstring,
+                            func,
+                            **v
+                        )
                 else:
-                    add_test(cls, test_name, func, v)
+                    add_test(cls, test_name, test_data_docstring, func, v)
             delattr(cls, name)
         elif hasattr(func, FILE_ATTR):
             file_attr = getattr(func, FILE_ATTR)
