@@ -147,8 +147,9 @@ def test_file_data_test_names_dict():
     tests_dir = os.path.dirname(__file__)
     test_data_path = os.path.join(tests_dir, 'test_data_dict.json')
     test_data = json.loads(open(test_data_path).read())
+    index_len = len(str(len(test_data)))
     created_tests = set([
-        "test_something_again_{0}_{1}".format(index + 1, name)
+        "test_something_again_{0:0{2}}_{1}".format(index + 1, name, index_len)
         for index, name in enumerate(test_data.keys())
     ])
 
@@ -236,14 +237,76 @@ def test_ddt_data_name_attribute():
     assert_is_not_none(getattr(ddt_mytest, 'test_hello_2_2'))
 
 
+def test_ddt_data_doc_attribute():
+    """
+    Test the ``__doc__`` attribute handling of ``data`` items with ``ddt``
+    """
+
+    def func_w_doc():
+        """testFunctionDocstring {6}
+
+        :param: None
+        :return: None
+        """
+        pass
+
+    def func_wo_doc():
+        pass
+
+    class Myint(int):
+        pass
+
+    class Mytest(object):
+        pass
+
+    d1 = Myint(1)
+    d1.__name__ = 'case1'
+    d1.__doc__ = """docstring1"""
+
+    d2 = Myint(2)
+    d2.__name__ = 'case2'
+
+    data_hello = data(d1, d2, {'test': True})(func_w_doc)
+    data_hello2 = data(d1, d2, {'test': True})(func_wo_doc)
+
+    setattr(Mytest, 'first_test', data_hello)
+    setattr(Mytest, 'second_test', data_hello2)
+    ddt_mytest = ddt(Mytest)
+
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'first_test_1_case1'), '__doc__'), d1.__doc__
+    )
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'first_test_2_case2'), '__doc__'),
+        func_w_doc.__doc__
+    )
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'first_test_3'), '__doc__'),
+        func_w_doc.__doc__
+    )
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'second_test_1_case1'), '__doc__'), d1.__doc__
+    )
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'second_test_2_case2'), '__doc__'),
+        None
+    )
+    assert_equal(
+        getattr(
+            getattr(ddt_mytest, 'second_test_3'), '__doc__'),
+        None
+    )
+
+
 def test_ddt_data_unicode():
     """
     Test that unicode strings are converted to function names correctly
     """
-
-    def hello():
-        pass
-
     # We test unicode support separately for python 2 and 3
 
     if six.PY2:
